/******************************************************************************
 * xtalk - an X implementation of talk
 Copyright (C) 1998  by Christopher Lansdown

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/

#include "callbacks.h"

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern GtkWidget *mainbox;
extern GtkWidget *text1, *text2, *text3;
extern GtkWidget *pane;
extern GtkWidget *input;
extern int sockfd;
extern int connected;

/* End Global Variables */
/*****************************************************************************/

int count_chars(char target, char *string);

void insert_callback(GtkEditable *editable, const gchar *text, gint length, gint *position)
{
    char info[32];
    char *tmp_text;

    snprintf(info, 32, "@i@%i@%i@", *position, length);
    write(sockfd, info, strlen(info));

    write(sockfd, text, length);

    return;
}

void delete_callback(GtkEditable *editable, gint start_pos, gint end_pos)
{
    char info[16];

    snprintf(info, 16, "@d@%i@%i@", start_pos, end_pos - start_pos);
    write(sockfd, info, strlen(info));

    return;
}

void comm_callback(gpointer data, gint fd, GdkInputCondition condition)
{
    char text[32];
    char format[33];
    int pos=-1, len=-1, cur_len=0, loc = 5, i;
    int count=0;
    char *txt;
    char *chr;

    bzero(text, 32);
    bzero(format, 33);

    while(count_chars('@', text) < 4 && count++ < 32)
	read(sockfd, &text[strlen(text)], 31-strlen(text));

    

    if(text3) {
	gtk_text_insert(GTK_TEXT (text3), NULL, NULL, NULL, text, strlen(text));
    }


    if(text[1] == 'i') {
	if(sscanf(text, "@i@%i@%i@", &pos, &len) == 2) {
	    if(pos >= 0 && len >= 0) {
		txt = malloc(len + 1);
		txt[len] = '\0';

		chr = text;
		for(i = 0; i < 3; i++)
		    chr = strchr(&chr[1], '@');

		if(chr) {
		    loc = (int)(chr - text);
		    loc++;
		    if((31 - loc) > len)
			cur_len = len;
		    else
			cur_len = 31 - loc;
		    strncpy(txt, &text[loc], cur_len);

		    cur_len = strlen(txt);
		
		    if(cur_len < len) {
			read(sockfd, &txt[cur_len], len - cur_len); 
			if(text3)
			    gtk_text_insert(GTK_TEXT (text3), NULL, 
					    NULL, NULL, txt, strlen(txt));
		    }
		    
		    
		    gtk_text_set_point(GTK_TEXT (text1), pos);
		    gtk_text_insert(GTK_TEXT (text1), 
				    NULL, NULL, NULL, txt, len);
		} else {
#ifdef DEBUG
		    printf("Bad insert format, not enough @ characters. \"%s\"\n", text);
		    fflush(NULL);
#endif
		}

		if(txt)
		    free(txt);
	    }
	}else {
#ifdef DEBUG
/* 	    printf("Error! bad insert format! \"%s\"\n", text); */
#endif
	}
    } else {
	if(sscanf(text, "@d@%i@%i@", &pos, &len) == 2) {
	    if(pos >= 0 && len >= 0) {
		gtk_text_set_point(GTK_TEXT (text1), pos);
		gtk_text_forward_delete(GTK_TEXT (text1), len);
	    }
	} else {
#ifdef DEBUG
/* 	    printf("Error! bad delete format! \"%s\"\n", text); */
#endif
	}
    }
    


    return;
}

int count_chars(char target, char *string)
{
    int cnt=0;
    char *chr=NULL;

    chr = strchr(string, target);

    while(chr != NULL) {
	cnt++;
	chr = strchr(&chr[1], target);
    }

    return cnt;
}

void input_callback(GtkWidget *widget, GtkWidget *entry)
{
    gchar *entry_text;
    char text[32];



    entry_text = gtk_entry_get_text(GTK_ENTRY (entry));
    if(strstr(entry_text, "/Quit") || strstr(entry_text, "/quit") || strstr(entry_text, "/exit") || strstr(entry_text, "/Exit"))
       {
	 sprintf(text, "Connection Closed.\n\n%c", (char)-127);
	 write(sockfd, "Connection Closed.\n\n", 20);
	 gtk_main_quit();
	 return;
       }

    /* Don't do anything if not connected */
    if(!connected)
      return;

    /* Output the string to the text-based screen */
    write(sockfd, entry_text, strlen(entry_text));
    write(sockfd, "\n", 1);

    /* Output the string to the X based screen */
    gtk_text_insert (GTK_TEXT (text2), NULL, NULL, NULL, entry_text, -1);
    gtk_text_insert (GTK_TEXT (text2), NULL, NULL, NULL, "\n", -1);
    
    /* Reset the input */
    gtk_entry_set_text(GTK_ENTRY (entry), "");
    return;
}


void text_callback(GtkWidget *widget, int fd)
{
    char text[1025];

    bzero(text, 1025);
    read(fd, text, 1024);

    if(text[0] == '\b')
      gtk_text_backward_delete(GTK_TEXT (text1), 1);
    else
      gtk_text_insert(GTK_TEXT (text1), NULL, NULL, NULL, text, -1);

    if(strchr(text, (char)-127)) {
      nice(20);
      printf("I got the term signal.\n");
      gdk_input_remove(fd);
      close(fd);
      connected = 0;
    }
      

    return;
}



int keyevent(GtkWidget *widget, GdkEventKey *event)
{
  /*if (event->keyval == 'a') do_stuff();*/
  char key[2];
  char text[32];


  key[0] = event->keyval;
  key[1] = '\0';

  /* Ok, I will use the appropriate negative numbers to signify the arrow keys and delete key */

  if(event->keyval == GDK_Up) {
    key[0] = -2;
    return TRUE;
  }
  if(event->keyval == GDK_Down) {
    key[0] = -3;
    return TRUE;
  }
  if(event->keyval == GDK_Right) {
    key[0] = -4;
    return TRUE;
  }
  if(event->keyval == GDK_Left) {
    key[0] = -5;
    return TRUE;
  }


  /* Filter the keys to get what we want */
  if(key[0] == 13)
    key[0] = '\n';

  
  if((key[0] == 'q' || key[0] == 'Q') && 
      (((event->state & GDK_CONTROL_MASK) ||
      (event->state & GDK_MOD1_MASK)))) {
    sprintf(text, "Connection Closed.\n\n%c", (char)-127);
    write(sockfd, text, 21);
    gtk_main_quit();
    return TRUE;
  }

  /* Don't do anything if not connected */
  if(!connected)
    return TRUE;
  
  /* If iti is negative, it is probably a control key */
  if(key[0] < -1)
    return TRUE;

  /* Handle the key */
  write(sockfd, key, 1);

  if(key[0] == '\b')
    gtk_text_backward_delete(GTK_TEXT (text2), 1);
  else if(key[0] == -1)
    gtk_text_forward_delete(GTK_TEXT (text2), 1);
  else
    gtk_text_insert (GTK_TEXT (text2), NULL, NULL, NULL, key, -1);

  return TRUE;
}
