/*****************************************************************************
   Xtalkc.c - the client for XTalk 
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "xtalk.h"
#include "callbacks.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

GtkWidget *window;
GtkWidget *mainbox;
GtkWidget *text1, *text2, *text3=NULL;
GtkWidget *pane;
GtkWidget *input;
int sockfd;
int connected;

/* End Global Variables */
/*****************************************************************************/

int main(int argc, char **argv)
{
    /* Begin Variable Declarations */
    GtkWidget *table;
    GtkWidget *scrollbar;
    char *addr, *tmp;
    /* End Variable Declarations */
    
    gtk_init(&argc, &argv);
    
    /* process the arguments */
    if(argc < 2) {
	printf("Usuage: %s [server]\n", argv[0]);
	exit(1);
    }
    else {
	if((tmp = strrchr(argv[1], '@')))
	    addr = tmp + 1;
	else
	    addr = argv[1];
    }
    
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "XTalkMe Client");
    gtk_widget_set_usize(GTK_WIDGET (window), 640, 480);
    gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
    gtk_signal_connect(GTK_OBJECT (window), "delete_event", 
		       (GtkSignalFunc) gtk_main_quit, NULL);
    signal(SIGINT, (void *)gtk_main_quit);
/*     gtk_signal_connect (GTK_OBJECT (window), "key_press_event", */
/* 			GTK_SIGNAL_FUNC(keyevent), NULL); */


    mainbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(mainbox);
    
    gtk_container_add(GTK_CONTAINER (window), mainbox);
    
    pane = gtk_vpaned_new();
    gtk_widget_show(pane);
    gtk_box_pack_start(GTK_BOX (mainbox), pane, TRUE, TRUE, 0);
    
    /* Set up the terminal text box */
    table = gtk_table_new(2, 2, FALSE);
    text1 = gtk_text_new(NULL, NULL);
    gtk_table_attach(GTK_TABLE (table), text1, 0, 1, 0, 1,
		     GTK_FILL | GTK_EXPAND,
		     GTK_FILL | GTK_EXPAND | GTK_SHRINK, 
		     0, 0);
    gtk_widget_show(text1);
    scrollbar = gtk_vscrollbar_new(GTK_TEXT(text1)->vadj);
    gtk_table_attach(GTK_TABLE (table), scrollbar, 1, 2, 0, 1,
		     GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 2);
    gtk_widget_show(scrollbar);
    gtk_widget_show(table);
    gtk_paned_add1(GTK_PANED (pane), table);
    
    /* Set up the X text box */
    table = gtk_table_new(2, 2, FALSE);
    text2 = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT (text2), TRUE);
    gtk_signal_connect(GTK_OBJECT(text2), "insert_text",
		       GTK_SIGNAL_FUNC (insert_callback), NULL);
    gtk_signal_connect(GTK_OBJECT(text2), "delete_text",
		       GTK_SIGNAL_FUNC (delete_callback), NULL);
    
    
    gtk_table_attach(GTK_TABLE (table), text2, 0, 1, 0, 1,
		     GTK_FILL | GTK_EXPAND,
		     GTK_FILL | GTK_EXPAND | GTK_SHRINK, 
		     0, 0);
    gtk_widget_show(text2);

    scrollbar = gtk_vscrollbar_new(GTK_TEXT(text2)->vadj);
    gtk_table_attach(GTK_TABLE (table), scrollbar, 1, 2, 0, 1,
		     GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 2);
    gtk_widget_show(scrollbar);
    gtk_widget_show(table);
    gtk_paned_add2(GTK_PANED (pane), table);
    
    
    /* Add the input entry area */
    /*
      input = gtk_entry_new();
      gtk_widget_show(input);
      gtk_entry_set_editable(GTK_ENTRY (input), TRUE);
      gtk_box_pack_start(GTK_BOX(mainbox), input, TRUE, TRUE, 0);
      gtk_signal_connect( GTK_OBJECT (input), "activate", GTK_SIGNAL_FUNC (input_callback), input);
    */
    
    gtk_widget_show(window);
    gtk_widget_grab_focus(GTK_WIDGET (text2));
    
    
    /* Connect to the server */
    sockfd = connect_to_server(addr, 3333);
    if(sockfd != -1)
/* 	gdk_input_add(sockfd, GDK_INPUT_READ, (GdkInputFunction) text_callback, (void *)sockfd); */
    gdk_input_add(sockfd, GDK_INPUT_READ, (GdkInputFunction) comm_callback, NULL);
    else {
	printf("Error: Unable to connect to Server!\n");
	exit(1);
    }
    
    connected = 1;
    
    gtk_main();
    return 0;
    
    
}

#ifdef DOG
#ifndef DOG
void create_get_server()
{
    GtkWidget *query;
    GtkWidget *label;
    GtkWidget *text;
    GtkWidget *ok;
    GtkWidget *cancel;
    
    query = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(query), "Select Host");
    gtk_widget_set_usize(GTK_WIDGET (query), 640, 480);
    gtk_signal_connect(GTK_OBJECT (query), "delete_event", (GtkSignalFunc) gtk_widget_destroy, query);
}
#endif
#endif

int connect_to_server(char *address, int port)
{
    int newsockfd;
    struct hostent *hostptr;
    struct in_addr inad;
    struct sockaddr_in serv_addr;
    
    if (inet_aton(address, &inad)) {
	hostptr = gethostbyaddr((char*)&inad, sizeof(inad), AF_INET);
    } 
    else {
	hostptr = gethostbyname(address);
    }
    
    if (!(hostptr && hostptr->h_name && hostptr->h_addr_list)) {
	printf("Invalid TCP/IP address: %s.", address);
	return -1;
    }
    
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family      = hostptr->h_addrtype;
    serv_addr.sin_port        = htons(port);
    bcopy(hostptr->h_addr, (char*) &serv_addr.sin_addr, hostptr->h_length);
    
    if ( (newsockfd = socket(AF_INET, SOCK_STREAM, 6)) < 0) {
	printf("Could not open stream socket %s.", address);
	return -1;
    }
    
    if (connect(newsockfd, (struct sockaddr *) &serv_addr,
		sizeof(serv_addr)) < 0) {
	printf("Client: could not connect to %s:%d", 
	       address, port);
	return -1;
    }
    
    return newsockfd;
}



